#!/usr/bin/perl

use Getopt::Long;

sub read_opts {
    my($usage); 

    $usage = "Usage: trmtx.pl -opt ..\n";
    $usage .= " -i input file\n";
    $usage .= " -o output file\n";
    $usage .= " -n number of \\internotes \n";
    $usage .= " [ -d down ]\n";

    &GetOptions('i=s','o=s','n=i','d') || die "$usage\n";
    ($opt_i && $opt_o && $opt_n) || die "$usage\n";

    $intv = $opt_n;

    # Make number of \internotes to be transposed negative
    # in case of downwards transposition
    $intv *= -1 if ($opt_d); 
} # sub read_opts

sub init_globals {

  # Convert from diatonic note names to base 7 numbers
  %note_number = 
    ("c","0",
     "d","1",
     "e","2",
     "f","3",
     "g","4",
     "a","5",
     "b","6");

  # Convert from base 7 numbers to diatonic note names
  %note = 
    ("0","c",
     "1","d",
     "2","e",
     "3","f",
     "4","g",
     "5","a",
     "6","b");
} # sub init_globals

sub print_paragraph {
    my(@mtx_pgf) = @_;
    my($line);
    
    foreach $line (@mtx_pgf) {
	print OUTFILE "$line\n";
    } # foreach $line (@mtx_pgf)

} # sub print_paragraph

sub transposed_line {
    my($line,$is_lyrics) = @_;
    my(@line, $word, $transposed_line);
    my($next_is_lyrics) = 0;

    # if current line contains lyrics label the following line is
    # a lyrics line. In that case leave current and the following line
    # unchanged.

    # Transpose note- or chord lines, leave other lines unchanged

    if ($is_lyrics) {
      $transposed_line = $line;
    }
    elsif (($line =~ m/^(\{\w+\}){1,}$/)) {
      $transposed_line = $line;
      $next_is_lyrics = 1;
    }
    elsif (($line =~ m/^c:/i) || (($line !~ m/^\w+:/) && ($line !~ m/^%/))) {
      # split line into words
      @line = split(/ /,$line);

      # Do necessary transposition on each word
      foreach $word (@line) {
	# Separate note values in dotted words
	($word,$word2) = split(/\./,$word);
	$word = transposed_word($word);
	# If dotted word transposesecond note value too
	if ($word2) {
	  $word2 = transposed_word($word2);
	  # re-concatenate dotted word
	  $word = "$word.$word2";
	}
	$transposed_line .= "$word ";
      }
      # Remove trailing added blank
      chop($transposed_line);
    }
    else {
      $transposed_line = $line;
    }

    ($transposed_line,$next_is_lyrics);
} # sub transposed_line

sub transposed_word {
  my($word) = @_;
  my($note_sym,$note_num,$note,$chord_pre,$note_suf);

  # Do transposition on note words only and leave the rest unchanged
  if (($word =~ m/^[a-g]/) || ($word =~ m/^z[a-g]/)) {
    # Separate chord note symbol prefix from remaining suffix and save both
    ($note_sym = $word) =~ s/(^z) *(.*)/$2/;
    $chord_pre = $1;

    # Separate note name prefix from remaining suffix and save both
    $note_sym =~ s/(^[a-g]) *(.*)/$1/;
    $note_suf = $2;

    # Convert note name to its note number 
    # and add number of \internotes to be transposed
    $note_num = $note_number{"$note_sym"} + $intv;

    # Add enough base 7 carries to make transposed note number positive
    while ($note_num < 0) {
      $note_num+=7;
    }

    # Reduce transposed positive note number using modulo 7
    $note_num = $note_num%7;

    # Convert transposed note number back to note name
    $note_sym = $note{"$note_num"};

    # Concatenate chord prefix, transposed note name and remaining note word
    $note_sym = sprintf("%s%s%s",$chord_pre,$note_sym,$note_suf);
  } # if (($word =~ m/^[a-g]/) || ($word =~ m/^z[a-g]/))
  else {
    $note_sym = $word;
  }
  
  $note_sym;
} # sub transposed_word

#
# Main program
#

read_opts;
init_globals;

open (INFILE, "<$opt_i") || die "Can't open $opt_i\n";
open (OUTFILE, ">$opt_o") || die "Can't open $opt_o\n";
#open (LOGFILE, ">trmtx.log") || die "Can't open pmx2mtx.log\n";

$next_is_lyrics = 0;
while (<INFILE>) {
  # Remove EOL, trailing and ending blanks
  # chop; s/^[ ]*//; s/[ ]*$//; 
    chop; 
    
    ($line,$next_is_lyrics) = transposed_line($_,$next_is_lyrics);
    print OUTFILE "$line\n";

} # while (<INFILE>)

exit;
